import json
from concurrent import futures

import grpc
import pydash

import llnw_pb2_grpc
from ll_sdk.config_api import ConfigApiClient
from llnw_pb2 import *


def get_cl(base):
    return ConfigApiClient('apis.llnw.com', base.username, base.key)


def delivery_to_grpc(delivery):
    return DeliveryEntity(
        uuid=delivery.get('uuid', None),
        isEnabled=delivery.get('isEnabled', None),
        shortname=delivery.get('shortname', None),
        status=pydash.objects.get(delivery, "status.state"),
        sourceHostname=pydash.objects.get(delivery, "body.sourceHostname"),
        publishedHostname=pydash.objects.get(delivery, "body.publishedHostname"),
        sourceUrlPath=pydash.objects.get(delivery, "body.sourceUrlPath"),
        publishedUrlPath=pydash.objects.get(delivery, "body.publishedUrlPath"),
        protocolSets=[ProtocolSet(
            sourceProtocol=protocolSets.get('sourceProtocol', None),
            publishedProtocol=protocolSets.get('publishedProtocol', None),
            options=[ProtocolSetOptions(
                name=options.get('name', None),
                parameters=[str(param) for param in options.get('parameters', None)],
                source=options.get('source', None),
            ) for options in protocolSets.get('options', [])]
        ) for protocolSets in pydash.objects.get(delivery, 'body.protocolSets', [])]
    )


def delivery_from_grpc(delivery_entity: DeliveryEntity):
    return {
        "uuid": delivery_entity.uuid,
        "accounts": [
            {
                "shortname": delivery_entity.shortname
            }
        ],
        "body": {
            "description": "由 python API 提交",
            "protocolSets": [
                {
                    "options": [
                        {
                            "name": po.name,
                            "parameters": [
                                para if po.name != "refresh_absmin" and po.name != "refresh_absmax" else int(para)
                                for para in po.parameters],
                            "source": po.source
                        } for po in protocol.options
                    ],
                    "publishedProtocol": protocol.publishedProtocol,
                    "sourceProtocol": protocol.sourceProtocol
                } for protocol in delivery_entity.protocolSets
            ],
            "publishedHostname": delivery_entity.publishedHostname,
            "publishedUrlPath": delivery_entity.publishedUrlPath,
            "serviceKey": {
                "name": "delivery"
            },
            "serviceProfileName": "LLNW-Generic",
            "serviceProfileVersion": 2,
            "sourceHostname": delivery_entity.sourceHostname,
            "sourceUrlPath": delivery_entity.sourceUrlPath
        },
        "meta": {
            "allowManualDuplicate": True
        }
    }


def check_error(res, context):
    if not res.get('success', True):
        match res.get('errorType'):
            case 'validation':
                context.abort(grpc.StatusCode.INVALID_ARGUMENT, json.dumps(res.get('errors')))
            case _:
                context.abort(grpc.StatusCode.UNKNOWN, json.dumps(res))


class DeliveryServiceServicer(llnw_pb2_grpc.DeliveryServiceServicer):

    def ListDelivery(self, request, context):
        res = get_cl(request.base).list_delivery_service_instances(request.shortName).json()
        check_error(res, context)
        return DeliveryList(deliveries=[delivery_to_grpc(results) for results in res['results']])

    def ValidateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        del delivery_config['uuid']
        res = get_cl(request.base).validate_delivery_service_instance(delivery_config).json()
        check_error(res, context)
        return delivery_to_grpc(res)

    def CreateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        del delivery_config['uuid']
        res = get_cl(request.base).create_delivery_service_instance(delivery_config).json()
        check_error(res, context)
        return delivery_to_grpc(res)

    def GetDelivery(self, request, context):
        res = get_cl(request.base).get_delivery_service_instance(request.id).json()
        check_error(res, context)
        return delivery_to_grpc(res)

    def UpdateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        res = get_cl(request.base).update_delivery_service_instance(delivery_config['uuid'], delivery_config).json()
        check_error(res, context)
        return delivery_to_grpc(res)

    def DeleteDelivery(self, request, context):
        res = get_cl(request.base).delete_delivery_service_instance(request.id).json()
        check_error(res, context)
        return delivery_to_grpc(res)


if __name__ == '__main__':
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    llnw_pb2_grpc.add_DeliveryServiceServicer_to_server(DeliveryServiceServicer(), server)
    server.add_insecure_port('[::]:50052')
    server.start()
    server.wait_for_termination()
